export function unserializeObject(serialized) {
    const result = {};
    if (typeof serialized !== 'object')
        throw new Error('The Argument is invalid, please make sure you are passing an object as the argument.');
    Object.entries(serialized).forEach(([key, value]) => {
        const keys = key.split(/[\.\[\]]+/).filter(Boolean);
        let current = result;
        keys.forEach((k, index) => {
            const isArray = index < keys.length - 1 && /^[\d]+$/.test(keys[index + 1]);
            if (index === keys.length - 1) {
                current[denormalizePrimitiveKey(k)] = valueParserByKey(k, value);
            }
            else {
                if (!current[k]) {
                    current[k] = isArray ? [] : {};
                }
                current = current[k];
            }
        });
    });
    return result;
}
export function serializeObject(unserialized) {
    const acc = {};
    const recursive = (data, parentKey = '') => {
        if (Array.isArray(data)) {
            data.forEach((item, index) => {
                const currentKey = parentKey ? `${parentKey}[${index}]` : '';
                recursive(item, currentKey);
            });
        }
        else if (typeof data === 'object' && data !== null) {
            Object.entries(data).forEach(([key, value]) => {
                const currentKey = parentKey ? `${parentKey}.${key}` : key;
                recursive(value, currentKey);
            });
        }
        else
            acc[normalizePrimitiveKey(parentKey, data)] = data;
    };
    if (typeof unserialized !== 'object')
        throw new Error('The Argument is invalid, please make sure you are passing an object as the argument.');
    recursive(unserialized);
    return acc;
}
export function normalizePrimitiveKey(key, value) {
    switch (typeof value) {
        case 'number':
            return `${key}()`;
        case 'boolean':
            return `${key}!!`;
        default:
            return key;
    }
}
export function denormalizePrimitiveKey(key) {
    const suffixes = ['()', '!!'];
    for (const suffix of suffixes) {
        if (key.endsWith(suffix)) {
            return key.slice(0, -suffix.length);
        }
    }
    return key;
}
export function valueParserByKey(key, value) {
    if (key.endsWith('()'))
        return Number(value);
    else if (key.endsWith('!!'))
        return Boolean(value);
    return value;
}
